local _, NS = ...
NS.PlayerControl = NS.PlayerControl or {}

local _print_debug_info = function(unused_event_name, ...)
  if not NS.Config.db.profile.settings.settings.debug then
    return
  end

  NS.print('===== Debug Loss of Control Info =====')

  NS.print('Event: ' .. unused_event_name)
  NS.print('arg:')
  NS.print(arg)
  NS.print('to_string(arg): ' .. NS.Utils.to_string(arg))

  local loc_data_count = C_LossOfControl.GetActiveLossOfControlDataCount()
  NS.print('loc_data_count: ' .. loc_data_count)
  for i = 1, loc_data_count do
    local loc_data = C_LossOfControl.GetActiveLossOfControlData(i)
    NS.print('loc_data:')
    NS.print(loc_data)
    NS.print('to_string(loc_data): ' .. NS.Utils.to_string(loc_data))
    -- Loss of Control debuffs that are applied only while standing in an
    -- Area of Effect may not include a startTime, timeRemaining nor duration
    -- in the table returned
  end

  NS.print('===== Done =====')
end

local _should_announce = function(loc_type)
  local effect_type_settings =
      NS.Config.db.profile.settings.settings.effect_types
  -- Effect types listed here:
  -- https://authors.curseforge.com/forums/world-of-warcraft/general-chat/lua-code-discussion/226080-stunned
  if loc_type == 'CONFUSE' then
    return effect_type_settings.confuse
  elseif loc_type == 'DISARM' then
    return effect_type_settings.disarm
  elseif loc_type == 'FEAR' then
    return effect_type_settings.fear
  elseif loc_type == 'PACIFY' then
    return effect_type_settings.pacify
  elseif loc_type == 'PACIFYSILENCE' then
    return effect_type_settings.pacify or
        effect_type_settings.silence
  elseif loc_type == 'POSSESS' then
    return effect_type_settings.possess
  elseif loc_type == 'ROOT' then
    return effect_type_settings.root
  elseif loc_type == 'SCHOOL_INTERRUPT' then
    return effect_type_settings.school_interrupt
  elseif loc_type == 'SILENCE' then
    return effect_type_settings.silence
  elseif loc_type == 'CHARM' or
      loc_type == 'STUN' or
      loc_type == 'STUN_MECHANIC' then
    return effect_type_settings.stun
  else
    NS.print(
        'You found a Loss of Control Effect Type that\'s not supported! ' ..
            'Please contact Alpheus-OldBlanchy and let him know! ' ..
            '(Include the following info:)\n' ..
            'locType: ' .. loc_data.locType .. '\n' ..
            'spellID: ' .. loc_data.spellID)
    return false
  end
end

local _get_chat_type = function()
  local announce_channel_settings =
      NS.Config.db.profile.settings.settings.announce_channels
  -- SendChatMessage is partially hw event protected:
  --   - "CHANNEL" is protected
  --   - "SAY", "YELL" are protected while outside of instances/raids
  -- https://wowpedia.fandom.com/wiki/Patch_8.2.5/API_changes
  -- https://twitter.com/deadlybossmods/status/1176685822223011842
  local is_in_instance = IsInInstance()
  if announce_channel_settings.battleground and
      UnitInBattleground('player') then
    return 'BATTLEGROUND'
  elseif announce_channel_settings.raid and IsInRaid() then
    return 'RAID'
  elseif announce_channel_settings.party and IsInGroup() then
    return 'PARTY'
  elseif announce_channel_settings.yell and is_in_instance then
    return 'YELL'
  elseif announce_channel_settings.say and is_in_instance then
    return 'SAY'
  elseif announce_channel_settings.system then
    return 'SYSTEM'
  elseif announce_channel_settings.print then
    return 'PRINT'
  else
    return nil
  end
end

local _get_school_text = function(school)
  if loc_data.lockoutSchool == STRING_SCHOOL_PHYSICAL then
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_PHYSICAL']
  elseif loc_data.lockoutSchool == STRING_SCHOOL_ARCANE then
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_ARCANE']
  elseif loc_data.lockoutSchool == STRING_SCHOOL_FIRE then
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_FIRE']
  elseif loc_data.lockoutSchool == STRING_SCHOOL_FROST then
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_FROST']
  elseif loc_data.lockoutSchool == STRING_SCHOOL_HOLY then
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_HOLY']
  elseif loc_data.lockoutSchool == STRING_SCHOOL_NATURE then
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_NATURE']
  elseif loc_data.lockoutSchool == STRING_SCHOOL_SHADOW then
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_SHADOW']
  else
    NS.print(
        'You found a School Interrupt type Loss of Control Effect with an ' ..
            'unknown school type! ' ..
            'Please contact Alpheus-OldBlanchy and let him know! ' ..
            '(Include the following info:)\n' ..
            'lockoutSchool: ' .. school)
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_UNKNOWN']
  end
end

local _get_display_text = function(loc_data)
  if loc_data.locType == 'SCHOOL_INTERRUPT' then
    local school_text = _get_school_text(loc_data.lockoutSchool)
    return NS.Locale['EFFECT_MESSAGE_SCHOOL_INTERRUPT_FN'](school)
  else
    return loc_data.displayText
  end
end

local _get_time_remaining_text = function(loc_data)
  -- From the documentation:
  -- https://wowpedia.fandom.com/wiki/API_C_LossOfControl.GetActiveLossOfControlData
  -- Loss of Control debuffs that are applied only while standing in an Area of
  -- Effect may not include a startTime, timeRemaining nor duration in the table
  -- returned.
  if loc_data.timeRemaining then
    local rounded_to_one_decimal =
        math.floor(loc_data.timeRemaining * 10 + 0.5) / 10
    return tostring(rounded_to_one_decimal)
  else
    return NS.Locale['EFFECT_MESSAGE_TIME_REMAINING_UNKNOWN']
  end
end

local _get_message = function(loc_data)
  return NS.Locale['EFFECT_MESSAGE_TEXT_FN'](
      _get_display_text(loc_data), _get_time_remaining_text(loc_data))
end

local _announce = function(loc_data)
  local chat_type = _get_chat_type()
  if not chat_type then
    return
  end
  local message = _get_message(loc_data)
  if chat_type == 'PRINT' then
    NS.print(message)
  elseif chat_type == 'SYSTEM' then
    SendSystemMessage(message)
  else
    SendChatMessage(message, chat_type)
  end
end

NS.PlayerControl.on_player_control_lost = function(unused_event_name, ...)
  _print_debug_info(unused_event_name, ...)
end

NS.PlayerControl.on_player_control_gained = function(unused_event_name, ...)
  _print_debug_info(unused_event_name, ...)
end

NS.PlayerControl.on_loss_of_control_added = function(unused_event_name, ...)
  _print_debug_info(unused_event_name, ...)
  local loc_data_count = C_LossOfControl.GetActiveLossOfControlDataCount()
  local max_time_remaining_loc_data = nil
  for i = 1, loc_data_count do
    local loc_data = C_LossOfControl.GetActiveLossOfControlData(i)
    if _should_announce(loc_data.locType) and
        (not max_time_remaining_loc_data or
         loc_data.timeRemaining > max_time_remaining_loc_data.timeRemaining)
        then
      max_time_remaining_loc_data = loc_data
    end
  end
  if not max_time_remaining_loc_data then
    return
  end
  _announce(max_time_remaining_loc_data)
end

NS.PlayerControl.on_loss_of_control_update = function(unused_event_name, ...)
  _print_debug_info(unused_event_name, ...)
end
